function EasyPet:OnInitialize()
    self.opt = self.db.profile
    
    EasyPetOptionsAnchor = CreateFrame("Frame", "EasyPetOptionsAnchor", UIParent)
    EasyPetOptionsAnchor:SetWidth(50); EasyPetOptionsAnchor:SetHeight(50); EasyPetOptionsAnchor:SetAlpha(0)
    EasyPetOptionsAnchor:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    EasyPetOptionsAnchor:EnableMouse(false); EasyPetOptionsAnchor:Hide()

    if EasyPetClassCheck() then
        -- LOAD BUTTONS
        EasyPetLoad_PetButtons()
        
        -- HIDE THEM ALL
        for i = 1, 8 do
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
        
        -- THEN SHOW THE CORRECT ONES
        EasyPet_UpdateButtons()

        -- FIX THE SIZE
        for i = 1, 5 do
            local button = getglobal("EasyPetButton"..i)
            button:SetScale(EasyPet.db.profile.Scale)
        end
        
        -- SHOW OR HIDE THE ANCHOR
        if not EasyPet.db.profile.Locked then EasyPetAnchor:EnableMouse(0); EasyPetAnchor:Hide() else EasyPetAnchor:EnableMouse(1); EasyPetAnchor:Show() end
    end
end

function EasyPet:OnEnable()
    self:RegisterEvent("UNIT_HEALTH")
    self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_BREAK_AURA")
    self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER")
    
    -- OPTIONS MENU
    for varname, val in pairs(EasyPet_Placement.args) do
        if self.opt[varname] then  EasyPet_Placement.args[varname].set(true) end
    end

    EasyPet.options:Register("EasyPetOptionsAnchor",
        'children', function(level, value) EasyPet:PopulateOptions(level, value) end,
        'point', function(parent) return "CENTER", "CENTER" end,
        'dontHook', true
    )
end

function EasyPet:PopulateOptions(level, value)
    if level == 1 then
        EasyPet.options:AddLine('text', "EasyPet", 'isTitle', true)
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine( 'text', Easy_Placement, 'hasArrow', true, 'value', Easy_Placement )
        EasyPet.options:AddLine( 'text', Easy_Scale, 'hasArrow', true, 'value', Easy_Scale )
        EasyPet.options:FeedAceOptionsTable(EasyPet_Drag, 0)
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine( 'text', Easy_ReadMe, 'hasArrow', true, 'value', Easy_ReadMe)
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine('text', Easy_Close, 'func', function() EasyPet.options:Close(1) end)

    elseif level == 2 then
        if value == Easy_Placement then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Placement, 1)
        elseif value == Easy_Scale then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Scale, 1)
        elseif value == Easy_ReadMe then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Notice, 1)
        end
    end
end

EasyPet:RegisterChatCommand("/easypet", "/ep", function() EasyPet.options:Open("EasyPetOptionsAnchor") end)

-- PLAYER TOOLTIP CHECK
function EasyPetTooltipCheck(spell)
    if IsMounted() then return true
    elseif UnitIsDeadOrGhost("player") then return true
    elseif not IsUsableSpell(spell) then return true
    end
end

-- RETURN CLASS
function EasyPetClassCheck()
    local _, class = UnitClass("player")
    if class == "WARLOCK" then return true else return false end
end

-- SCALE UPDATE
function EasyPetScale()
    for i = 1, 5 do
        local button = getglobal("EasyPetButton"..i)
        button:SetScale(EasyPet.db.profile.Scale)
    end
end

-- COOLDOWN UPDATE
function EasyPet:SPELL_UPDATE_COOLDOWN()
    if EasyPetClassCheck() then
        if (GetSpellCooldown(EasyDevour) ~= nil and GetSpellCooldown(EasyDevour) > 0) then
            local start, duration, enable = GetSpellCooldown(EasyDevour)
            EasyPetButton1Cooldown:SetCooldown(start, duration)
            EasyPetButton2Cooldown:SetCooldown(start, duration)
            EasyPetButton3Cooldown:SetCooldown(start, duration)
        elseif (GetSpellCooldown(EasySpellLock) ~= nil and GetSpellCooldown(EasySpellLock) > 0) then
            local start, duration, enable = GetSpellCooldown(EasySpellLock)
            EasyPetButton4Cooldown:SetCooldown(start, duration)
        elseif (GetSpellCooldown(EasySeduce) ~= nil and GetSpellCooldown(EasySeduce) > 0) then
            local start, duration, enable = GetSpellCooldown(EasySeduce);
            EasyPetButton5Cooldown:SetCooldown(start, duration);
        elseif (GetSpellCooldown(EasyIntercept) ~= nil and GetSpellCooldown(EasyIntercept) > 0) then
            local start, duration, enable = GetSpellCooldown(EasyIntercept)
            EasyPetButton8Cooldown:SetCooldown(start, duration)
        end
    end
end

-- FIND SPELL NUMBER
function EasyPet_SpellNumber(spell)
    local i = 1
    while true do
       local spellName, spellRank = GetSpellName(i, BOOKTYPE_PET)
       if not spellName then
          do break end
       end
       
       if (spellName == spell) then return i end
       
       i = i + 1
    end
end

-- UPDATE BUTTONS
function EasyPet_UpdateButtons()
    if (EasyPetDemonCheck() == EasyFelhunter) then
        EasyPetButton1:Show()
        EasyPetButton2:Show()
        EasyPetButton3:Show()
        EasyPetButton4:Show()
        
        for i = 5, 8 do
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (EasyPetDemonCheck() == EasySuccubus) then
        EasyPetButton5:Show()
        
        for i = 1, 8 do
            if i == 5 then break end
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (EasyPetDemonCheck() == EasyVoid) then
        EasyPetButton6:Show()
        EasyPetButton7:Show()
        
        for i = 1, 8 do
            if i == 6 then break end
            if i == 7 then break end
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (EasyPetDemonCheck() == EasyFelguard) then
        EasyPetButton8:Show()
        
        for i = 1, 7 do
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    end
    
    EasyPetScale()
end

function EasyPet:UNIT_HEALTH()
    if EasyPetClassCheck() then            
        if IsMounted() then
            local AnyButtons = false
            for i = 1, 8 do
                local button = getglobal("EasyPetButton"..i)
                if button then
                    if button:IsVisible() then AnyButtons = true end
                end
            end
            if not AnyButtons and UnitAffectingCombat("player") then
                EasyPetMsg:AddMessage(Easy_Error)
            end
            return
        end
        
        if not UnitAffectingCombat("player") then
            EasyPet_UpdateButtons()
        end
    end
end

function EasyPet:PLAYER_REGEN_ENABLED()
    if EasyPetClassCheck() then
        if not UnitAffectingCombat("player") then
            EasyPet_UpdateButtons()
        end
    end
end

-- PET FAMILY CHECK
function EasyPetDemonCheck()
    if UnitCreatureFamily("pet") then
        return UnitCreatureFamily("pet")
    end
end

-- SEDUCTION TIMER
function EasyPetSeductionTimer(time)
    if (EasyPetDemonCheck() == EasySuccubus) and string.find(string.lower(arg1), string.lower(EasySeduce)) then
        EasyPetButton5Cooldown:SetCooldown(GetTime(), time)
    end
end

function EasyPet:CHAT_MSG_SPELL_AURA_GONE_OTHER(arg1)
    EasyPetSeductionTimer(0)
end

function EasyPet:CHAT_MSG_SPELL_BREAK_AURA(arg1)
    EasyPetSeductionTimer(0)
end

function EasyPet:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE(arg1)
    EasyPetSeductionTimer(12)
end

function EasyPet:CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE(arg1)
    EasyPetSeductionTimer(15)
end